"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var typescript_1 = __importDefault(require("typescript"));
var const_1 = require("./const");
var type_1 = require("./type");
var formProperFilePath = function (props) {
    return props.filePath.split(const_1.separator).filter(Boolean).join(const_1.separator);
};
var checkJavaScriptFileExistByAppend = function (props) {
    var result = const_1.ExtensionsUtil.extensions.javaScript
        .map(function (extension) {
        return {
            extension: extension,
            filePath: "".concat(props.filePath).concat(extension),
        };
    })
        .find(function (filePath) {
        return fs_1.default.existsSync(filePath.filePath);
    });
    return result !== null && result !== void 0 ? result : false;
};
var checkTypeDefinitionFileExistByAppend = function (props) {
    var _a = const_1.ExtensionsUtil.extensions.javaScript, js = _a[0], mjs = _a[1];
    var _b = const_1.ExtensionsUtil.extensions.typeDefinition, dts = _b[0], mdts = _b[1];
    var dtsFilePath = "".concat(props.filePath).concat(dts);
    if (fs_1.default.existsSync(dtsFilePath)) {
        return { extension: js, filePath: dtsFilePath };
    }
    var mdtsFilePath = "".concat(props.filePath).concat(mdts);
    if (fs_1.default.existsSync(mdtsFilePath)) {
        return { extension: mjs, filePath: mdtsFilePath };
    }
    return false;
};
var isDirectory = function (filePath) {
    try {
        return fs_1.default.lstatSync(filePath).isDirectory();
    }
    catch (error) {
        return false;
    }
};
var addJSExtensionConditionally = function (props) {
    var check = props.checkType === 'js'
        ? checkJavaScriptFileExistByAppend
        : checkTypeDefinitionFileExistByAppend;
    var skip = {
        procedure: 'skip',
    };
    switch (isDirectory(props.filePath)) {
        case true: {
            var result = check({
                filePath: path_1.default.posix.join(props.filePath, 'index'),
            });
            if (!result) {
                return skip;
            }
            var file = "index".concat(result.extension);
            return {
                procedure: 'proceed',
                absolutePath: result.filePath,
                importPath: formProperFilePath({
                    filePath: "".concat(props.importPath).concat(const_1.separator).concat(file),
                }),
            };
        }
        case false: {
            var result = check({
                filePath: props.filePath,
            });
            if (!result) {
                return skip;
            }
            return {
                procedure: 'proceed',
                absolutePath: result.filePath,
                importPath: formProperFilePath({
                    filePath: "".concat(props.importPath).concat(result.extension),
                }),
            };
        }
    }
};
var addJSExtension = function (props) {
    if (const_1.ExtensionsUtil.matchJs(props.filePath)) {
        return {
            procedure: 'skip',
        };
    }
    var result = addJSExtensionConditionally(__assign(__assign({}, props), { checkType: 'js' }));
    switch (result.procedure) {
        case 'proceed': {
            return result;
        }
        case 'skip': {
            return addJSExtensionConditionally(__assign(__assign({}, props), { checkType: 'dts' }));
        }
    }
};
var traverseAndUpdateFileWithJSExtension = function (files) {
    return function (_a) {
        var parsed = _a.parsed, code = _a.code;
        var delimiter = '';
        var characters = code.split(delimiter);
        var file = parsed.fileName;
        var replaceNodes = parsed.statements.flatMap(function (statement) {
            switch (statement.kind) {
                case typescript_1.default.SyntaxKind.ImportDeclaration:
                case typescript_1.default.SyntaxKind.ExportDeclaration: {
                    var importExportDeclaration_1 = statement;
                    if (!importExportDeclaration_1.moduleSpecifier) {
                        return [];
                    }
                    var moduleSpecifier = (0, type_1.asString)({
                        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                        // @ts-ignore
                        value: importExportDeclaration_1.moduleSpecifier.text,
                        error: new Error('Module specifier of node should have text'),
                    });
                    var fileName = formProperFilePath({
                        filePath: !moduleSpecifier.endsWith(const_1.separator)
                            ? moduleSpecifier
                            : moduleSpecifier.slice(0, -1),
                    })
                        .split(const_1.separator)
                        .at(-1);
                    if (!fileName) {
                        throw new Error("Impossible for file name to be non-existent for ".concat(moduleSpecifier));
                    }
                    if (moduleSpecifier.startsWith('.')) {
                        var result_1 = addJSExtension({
                            importPath: moduleSpecifier,
                            filePath: path_1.default.posix.join(file, '..', moduleSpecifier),
                        });
                        switch (result_1.procedure) {
                            case 'proceed': {
                                // if file name not included in list of js file read
                                if (files.find(function (file) {
                                    return file.endsWith(result_1.absolutePath);
                                })) {
                                    var before = characters
                                        .filter(function (_, index) {
                                        return (index >
                                            importExportDeclaration_1.pos &&
                                            index <
                                                importExportDeclaration_1.end);
                                    })
                                        .join(delimiter);
                                    return [
                                        {
                                            before: before,
                                            after: before.replace(moduleSpecifier, result_1.importPath),
                                        },
                                    ];
                                }
                            }
                        }
                    }
                }
            }
            return [];
        });
        return !replaceNodes.length
            ? []
            : [
                {
                    file: file,
                    code: replaceNodes.reduce(function (code, node) {
                        return code.replace(node.before, node.after);
                    }, code),
                },
            ];
    };
};
exports.default = traverseAndUpdateFileWithJSExtension;
//# sourceMappingURL=traverse-and-update.js.map